"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const FS = require("fs");
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const devKitCore = require("@angular-devkit/core");
const build_ng_packagr_1 = require("@angular-devkit/build-ng-packagr");
const ngPackagr = require("ng-packagr");
// TODO: Remove when issue is fixed.
require("./workaround-issue-1189");
const create_hook_provider_1 = require("./create-hook-provider");
const utils_1 = require("./utils");
const DEFAULT_TSCONFIG_OPTIONS = {
    moduleResolution: 'node',
    module: 'commonjs',
    target: 'es6',
    lib: [
        'es2017',
        'dom'
    ],
};
// Instead of re-writing NgPackagrBuilder in full, just for adding some providers, we will patch
// the NgPackager instead... ohh my!
class NgPackagrBuilder extends build_ng_packagr_1.NgPackagrBuilder {
    run(builderConfig) {
        const builderContext = this.context;
        this.options = builderConfig.options;
        const root = this.context.workspace.root;
        const { tasks } = this.options;
        const { build, watch } = ngPackagr.NgPackagr.prototype;
        const globalTasksContext = {
            logger: this.context.logger,
            root,
            builderContext,
            builderConfig
        };
        return rxjs_1.from(this.getTransformerHooks(tasks.config, globalTasksContext))
            .pipe(operators_1.switchMap(transformerHooks => utils_1.validateTypedTasks(transformerHooks, this.context, tasks)), operators_1.tap(transformerHooks => {
            const providers = create_hook_provider_1.createHookProviders(transformerHooks, globalTasksContext);
            ngPackagr.NgPackagr.prototype.build = function () {
                this.withProviders(providers);
                return build.call(this);
            };
            ngPackagr.NgPackagr.prototype.watch = function () {
                this.withProviders(providers);
                return watch.call(this);
            };
        }), operators_1.switchMap(() => super.run(builderConfig)), operators_1.tap(buildEvent => {
            ngPackagr.NgPackagr.prototype.build = build;
            ngPackagr.NgPackagr.prototype.watch = watch;
        }));
    }
    getTransformerHooks(transformerPath, globalTasksContext) {
        const { tasks } = this.options;
        const root = this.context.workspace.root;
        const tPath = devKitCore.resolve(root, devKitCore.normalize(transformerPath));
        if (FS.existsSync(tPath)) {
            if (/\.ts$/.test(tPath) && !require.extensions['.ts']) {
                const tsNodeOptions = {};
                if (tasks.tsConfig) {
                    tsNodeOptions.project = tasks.tsConfig;
                }
                else {
                    tsNodeOptions.compilerOptions = DEFAULT_TSCONFIG_OPTIONS;
                }
                require('ts-node').register(tsNodeOptions);
            }
            const transformHooksModule = require(tPath);
            const transformHooks = typeof transformHooksModule === 'function'
                ? transformHooksModule(globalTasksContext)
                : transformHooksModule;
            return Promise.resolve(transformHooks).then(utils_1.normalizeHooks);
        }
        ;
        return Promise.resolve({});
    }
}
exports.NgPackagrBuilder = NgPackagrBuilder;
exports.default = NgPackagrBuilder;
//# sourceMappingURL=index.js.map